#include "graph_displayer_canvas.h"
#include <cmath>

GraphDisplayerCanvas::GraphDisplayerCanvas(NoteBookTab & mainw) :
  nodesmap(mainw.mapstorage.graph), edgesmap(mainw.mapstorage.graph), edgetextmap(mainw.mapstorage.graph),
  nodetextmap(mainw.mapstorage.graph), displayed_graph(*(root()), 0, 0),
  isbutton(0), active_item(NULL), target_item(NULL), nodemap_to_edit(""),
  edgemap_to_edit(""), mytab(mainw)
{
  //base event handler is move tool
  actual_handler=signal_event().connect(sigc::mem_fun(*this, &GraphDisplayerCanvas::moveEventHandler), false);
  actual_tool=MOVE;

  active_node=INVALID;
  active_edge=INVALID;
  forming_edge=INVALID;
}

GraphDisplayerCanvas::~GraphDisplayerCanvas()
{
  for (NodeIt n((mytab.mapstorage).graph); n != INVALID; ++n)
    {
      delete nodesmap[n];
      delete nodetextmap[n];
    }
  
  for (EdgeIt e((mytab.mapstorage).graph); e != INVALID; ++e)
    {
      delete edgesmap[e];
      delete edgetextmap[e];
    }
}

void GraphDisplayerCanvas::propertyChange(bool itisedge, int prop)
{
  if(itisedge)
    {
      propertyUpdate(Edge(INVALID), prop);
    }
  else
    {
      propertyUpdate(Node(INVALID), prop);
    }
}

void GraphDisplayerCanvas::propertyUpdate(Edge edge)
{
  for(int i=0;i<EDGE_PROPERTY_NUM;i++)
    {
      propertyUpdate(edge, i);
    }
}

void GraphDisplayerCanvas::propertyUpdate(Node node)
{
  for(int i=0;i<NODE_PROPERTY_NUM;i++)
    {
      propertyUpdate(node, i);
    }
}

void GraphDisplayerCanvas::propertyUpdate(Node node, int prop)
{
  //dummy=dummy;

  std::string mapname=mytab.getActiveNodeMap(prop);

  if(mapname!="")
    {
      if( ( ((mytab.mapstorage).nodemap_storage).find(mapname) != ((mytab.mapstorage).nodemap_storage).end() ) )
	{
	  switch(prop)
	    {
	    case N_RADIUS:
	      changeNodeRadius(mapname, node);
	      break;
	    case N_COLOR:
	      changeNodeColor(mapname, node);
	      break;
	    case N_TEXT:
	      changeNodeText(mapname, node);
	      break;
	    default:
	      std::cerr<<"Error\n";
	    }
	}
    }
  else //mapname==""
    {
      Node node=INVALID;	
      switch(prop)
	{
	case N_RADIUS:
	  resetNodeRadius(node);
	  break;
	case N_COLOR:
	  resetNodeColor(node);
	  break;
	case N_TEXT:
	  resetNodeText(node);
	  break;
	default:
	  std::cerr<<"Error\n";
	}
    }

}

void GraphDisplayerCanvas::propertyUpdate(Edge edge, int prop)
{
  //dummy=dummy;

  std::string mapname=mytab.getActiveEdgeMap(prop);

  if(mapname!="")
    {
      if( ( ((mytab.mapstorage).edgemap_storage).find(mapname) != ((mytab.mapstorage).edgemap_storage).end() ) )
	{
	  switch(prop)
	    {
	    case E_WIDTH:
	      changeEdgeWidth(mapname, edge);
	      break;
	    case E_COLOR:
	      changeEdgeColor(mapname, edge);
	      break;
	    case E_TEXT:
	      changeEdgeText(mapname, edge);
	      break;
	    default:
	      std::cerr<<"Error\n";
	    }
	}
    }
  else //mapname==""
    {
      switch(prop)
	{
	case E_WIDTH:
	  resetEdgeWidth(edge);
	  break;
	case E_COLOR:
	  resetEdgeColor(edge);
	  break;
	case E_TEXT:
	  resetEdgeText(edge);
	  break;
	default:
	  std::cerr<<"Error\n";
	}
    }
}

void GraphDisplayerCanvas::drawGraph()
{
  //first edges are drawn, to hide joining with nodes later

  for (EdgeIt i((mytab.mapstorage).graph); i!=INVALID; ++i)
  {
    if (mytab.mapstorage.graph.source(i) == mytab.mapstorage.graph.target(i))
    {
      edgesmap[i]=new LoopEdge(displayed_graph, i, *this);
    }
    else
    {
      edgesmap[i]=new BrokenEdge(displayed_graph, i, *this);
    }
    //initializing edge-text as well, to empty string

    XY text_pos=mytab.mapstorage.arrow_pos[i];
    text_pos+=(XY(10,10));

    edgetextmap[i]=new Gnome::Canvas::Text(displayed_graph, text_pos.x, text_pos.y, "");
    edgetextmap[i]->property_fill_color().set_value("darkgreen");
    edgetextmap[i]->signal_event().connect(sigc::mem_fun(*this, &GraphDisplayerCanvas::mapEditEventHandler), false);
    edgetextmap[i]->raise_to_top();
  }

  //afterwards nodes come to be drawn

  for (NodeIt i((mytab.mapstorage).graph); i!=INVALID; ++i)
  {
    //drawing bule nodes, with black line around them

    nodesmap[i]=new Gnome::Canvas::Ellipse(
        displayed_graph,
        (mytab.mapstorage).coords[i].x-20,
        (mytab.mapstorage).coords[i].y-20,
        (mytab.mapstorage).coords[i].x+20,
        (mytab.mapstorage).coords[i].y+20);
    *(nodesmap[i]) << Gnome::Canvas::Properties::fill_color("blue");
    *(nodesmap[i]) << Gnome::Canvas::Properties::outline_color("black");
    nodesmap[i]->raise_to_top();

    //initializing edge-text as well, to empty string

    XY text_pos(
        ((mytab.mapstorage).coords[i].x+node_property_defaults[N_RADIUS]+5),
        ((mytab.mapstorage).coords[i].y+node_property_defaults[N_RADIUS]+5));

    nodetextmap[i]=new Gnome::Canvas::Text(displayed_graph,
        text_pos.x, text_pos.y, "");
    nodetextmap[i]->property_fill_color().set_value("darkblue");
    nodetextmap[i]->signal_event().connect(sigc::mem_fun(*this, &GraphDisplayerCanvas::mapEditEventHandler), false);
    nodetextmap[i]->raise_to_top();
  }

  updateScrollRegion();
}

void GraphDisplayerCanvas::clear()
{
  active_node=INVALID;
  active_edge=INVALID;
  forming_edge=INVALID;

  for (NodeIt n((mytab.mapstorage).graph); n != INVALID; ++n)
  {
    delete nodesmap[n];
    delete nodetextmap[n];
  }

  for (EdgeIt e((mytab.mapstorage).graph); e != INVALID; ++e)
  {
    delete edgesmap[e];
    delete edgetextmap[e];
  }
}
